"""
Test main entry point
"""

import unittest

from middlefield import _main


# pylint: disable=assignment-from-none
class NoOpTest(unittest.TestCase):

    """
    Tests for noop
    """

    def test_noop(self):
        """
        noop returns None
        """
        res = _main.noop(["hello"])
        self.assertIsNone(res)
# pylint: enable=assignment-from-none



"""
Middlefield -- a multi-function build tool
"""

from middlefield._api import COMMANDS, self_build
from middlefield._version import __version__ as _raw_version

__version__ = _raw_version.short()

__all__ = ['COMMANDS', 'self_build', '__version__']



"""
Middlefield's main API
"""

import contextlib
import os
import typing
import shutil
import tempfile

from caparg import command, option
import elcaminoreal
from pex import pex_builder
import seashore

from middlefield import _version

COMMANDS = elcaminoreal.Commands()


@COMMANDS.dependency()
def executor(_dependencies, _maybe_dependencies):
    """
    Return an executor
    """
    return seashore.Executor(seashore.Shell())


@COMMANDS.dependency(name='pex_builder')
def get_pex_builder(_dependencies, _maybe_dependencies):
    """
    Return a Pex builder
    """
    return pex_builder.PEXBuilder


@contextlib.contextmanager
def tmpdir():
    """
    Context manager creating a temporary directory
    """
    ret = tempfile.mkdtemp()
    try:
        yield ret
    finally:
        shutil.rmtree(ret)


@COMMANDS.command(parser=command('',
                                 requirements=option(type=typing.List[str],
                                                     have_default=True),
                                 package=option(type=typing.List[str],
                                                have_default=True),
                                 shebang=option(type=str),
                                 output=option(type=str, required=True)),
                  dependencies=['executor', 'pex_builder'])
def self_build(args, dependencies):
    """
    Build middlefield, together with any plugins, into a Pex file
    """
    package = list(args['package'])
    my_version = _version.__version__.short()
    package.append('middlefield=={}'.format(my_version))
    requirements = list(args['requirements'])
    output = args['output']
    xctor = dependencies['executor']
    builder = dependencies['pex_builder']()
    builder.set_entry_point('middlefield')
    if 'shebang' in args:
        builder.set_shebang(args['shebang'])
    with tmpdir() as wheelhouse:
        xctor.pip.wheel(*list(package),
                        requirements=requirements,
                        wheel_dir=wheelhouse).batch()
        for dist in os.listdir(wheelhouse):
            dist = os.path.join(wheelhouse, dist)
            builder.add_dist_location(dist)
        builder.build(output)



"""
Run command using :code:`middlefield.COMMANDS`
"""

import sys

import elcaminoreal
import middlefield

if __name__ != '__main__':
    raise ImportError("module cannot be imported")

with elcaminoreal.errors_to(sys.stderr):  # pragma: no cover
    middlefield.COMMANDS.run(sys.argv[1:])



import setuptools

with open('README.rst') as fp:
    long_description = fp.read()

setuptools.setup(
    name='middlefield',
    license='MIT',
    description="Middlefield: A multifunctional tool.",
    long_description=long_description,
    use_incremental=True,
    setup_requires=['incremental'],
    author="Moshe Zadka",
    author_email="zadka.moshe@gmail.com",
    packages=setuptools.find_packages(where='src'),
    package_dir={"": "src"},
    install_requires=['elcaminoreal', 'incremental', 'caparg',
                      'seashore', 'pex', 'toolz'],
    entry_points=dict(
        gather=["gather=middlefield"],
        console_scripts=["mf=middlefield._main:entrypoint"],
    )
)
